//
//  WordlistViewController.m
//  iOS Hotspot Cracker
//
//  Copyright (c) 2013, Andreas Kurtz & Markus Trossbach, Friedrich-Alexander-University, Erlangen, Germany.
//  All rights reserved.
//

#import "WordlistViewController.h"
#import "WordlistGenerator.h"



@interface WordlistViewController ()

- (void)displayProgress:(BOOL)display;
- (void)displayGeneratorButton:(BOOL)display;
- (void)displayWordlistAvailable:(BOOL)display;

@end

@implementation WordlistViewController

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)dealloc {
    [[WordlistGenerator sharedInstance] removeObserver:self forKeyPath:@"progress"];
    [[WordlistGenerator sharedInstance] removeObserver:self forKeyPath:@"stage"];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    self.scrollView.contentSize = CGSizeMake(320, 455);

    
    if([WordlistGenerator sharedInstance].stage == kWordlistGenStageAvailable) {
        [self displayWordlistAvailable:YES];
        [self displayProgress:NO];
        [self displayGeneratorButton:NO];
    } else if ([WordlistGenerator sharedInstance].stage == kWordlistGenStageNotAvailable) {
        [self displayWordlistAvailable:NO];
        [self displayProgress:NO];
        [self displayGeneratorButton:YES];
    }
 
    [[WordlistGenerator sharedInstance] addObserver:self forKeyPath:@"progress" options:NSKeyValueObservingOptionNew context:nil];
    [[WordlistGenerator sharedInstance] addObserver:self forKeyPath:@"stage" options:NSKeyValueObservingOptionNew context:nil];
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context {
    if([keyPath isEqualToString:@"stage"]) {
        [self performSelectorOnMainThread:@selector(updateStage) withObject:nil waitUntilDone:NO];
    } else if ([keyPath isEqualToString:@"progress"]) {
        [self performSelectorOnMainThread:@selector(updateProgress) withObject:nil waitUntilDone:NO];
    }
}

- (void)updateStage {
    if([WordlistGenerator sharedInstance].stage == kWordlistGenStageAvailable) {
        [self displayWordlistAvailable:YES];
        [self displayProgress:NO];
        [self displayGeneratorButton:NO];
    } else {
        self.wordlistProgress.progress = 0.0f;
        [self displayWordlistAvailable:NO];
        [self displayProgress:YES];
        [self displayGeneratorButton:NO];
        
        if([WordlistGenerator sharedInstance].stage == kWordlistGenStageGeneratingWords) {
            self.stageLabel.text = @"Stage 1: Generating words ...";
        } else if ([WordlistGenerator sharedInstance].stage == kWordlistGenStageFinalizing) {
            self.stageLabel.text = @"Stage 2: Sorting and finalizing list ...";
        }
    }
}

- (void)updateProgress {
    self.wordlistProgress.progress = [WordlistGenerator sharedInstance].progress;    
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (IBAction)startGenerator:(id)sender {
        [[WordlistGenerator sharedInstance] start];
}

- (void)displayGeneratorButton:(BOOL)display {
    self.startGeneratorButton.hidden = !display;
}

- (void)displayProgress:(BOOL)display {
    self.wordlistProgress.hidden = !display;
    self.wordlistActivityIndicator.hidden = !display;
    if(display) {
        if(!self.wordlistActivityIndicator.isAnimating) {
            [self.wordlistActivityIndicator startAnimating];
        }
    } else {
        [self.wordlistActivityIndicator stopAnimating];
    }
    self.stageLabel.hidden = !display;
}

- (void)displayWordlistAvailable:(BOOL)display {
    self.exportButton.enabled = display;
    self.wordlistGeneratedLabel.hidden = !display;
    self.wordlistGeneratedLabel.text = [NSString stringWithFormat:@"Wordlist successfully generated, it consists of %i unique entries.", [[WordlistGenerator sharedInstance].sortedWordlist count]];
}

- (IBAction)tapExport:(id)sender {
    MFMailComposeViewController *mailController = [[MFMailComposeViewController alloc] init];
    if(mailController) {
        [mailController setSubject:@"iOS Hotspot Cracker Wordlist"];
        [mailController setMessageBody:@"Apple iOS generates weak default passwords to be used in its \"Personal Hotspot\" feature. This makes the Personal Hotspot susceptible to brute-force attacks on the WPA handshake.\n\nTo produce easy pronounceable words, the iOS spell checking service is used to extract words in the length of 4 to 6 characters followed by a four-digit number. As only 1.842 entries of that dictionary are taken into consideration, the default password of an arbitrary iOS Hotspot can be revealed within seconds.\n\nPlease find a generated \"Personal Hotspot Cracking Wordlist\" attached to this message." isHTML:NO];
        mailController.mailComposeDelegate = self;
        [mailController addAttachmentData:[WordlistGenerator sharedInstance].dataForWordlist mimeType:@"text/plain; charset=UTF-8" fileName:@"wordlist.txt"];
        [self presentViewController:mailController animated:YES completion:^{
            
        }];
    }
}

- (void)mailComposeController:(MFMailComposeViewController *)controller didFinishWithResult:(MFMailComposeResult)result error:(NSError *)error {
    [controller dismissViewControllerAnimated:YES completion:^{
        
    }];
}

@end
